@@ -331,7 +331,7 @@ generate_file_entry (cherokee_handler_dirlist_t *dhdl, DIR *dir, cherokee_buffer
 		 */
 		re = cherokee_lstat (path->buf, &n->stat);
 		if (re < 0) {
-			cherokee_buffer_drop_endding (path, n->name_len);
+			cherokee_buffer_drop_ending (path, n->name_len);
 
 			free (n);
 			return ret_error;
@@ -339,7 +339,7 @@ generate_file_entry (cherokee_handler_dirlist_t *dhdl, DIR *dir, cherokee_buffer
 
 		/* Clean up and exit
 		 */
-		cherokee_buffer_drop_endding (path, n->name_len);
+		cherokee_buffer_drop_ending (path, n->name_len);
 
 		*ret_entry = n;
 		return ret_ok;
@@ -356,6 +356,8 @@ cherokee_handler_dirlist_new  (cherokee_handler_t **hdl, void *cnt, cherokee_mod
 	char  *value;
 	CHEROKEE_NEW_STRUCT (n, handler_dirlist);
 	
+	TRACE_CONN(cnt);
+
 	/* Init the base class object
 	 */
 	cherokee_handler_init_base (HANDLER(n), cnt, HANDLER_PROPS(props), PLUGIN_INFO_HANDLER_PTR(dirlist));
@@ -450,27 +452,44 @@ cherokee_handler_dirlist_free (cherokee_handler_dirlist_t *dhdl)
 static ret_t
 check_request_finish_with_slash (cherokee_handler_dirlist_t *dhdl)
 {
+	cuint_t                len;
 	cherokee_connection_t *conn = HANDLER_CONN(dhdl);
 
 	if ((cherokee_buffer_is_empty (&conn->request)) ||
-	    (!cherokee_buffer_is_endding (&conn->request, '/'))) {
-
+	    (!cherokee_buffer_is_ending (&conn->request, '/'))) 
+	{
+		/* Build the redirection address
+		 */
 		cherokee_buffer_clean (&conn->redirect);
-		cherokee_buffer_ensure_size (&conn->redirect, 
-					     conn->web_directory.len + conn->request.len + conn->userdir.len + 4);
+
+		len = conn->web_directory.len + 
+		      conn->request.len + 
+		      conn->userdir.len + 
+		      conn->host.len + 
+		      sizeof("https://") + 4;
+
+		cherokee_buffer_ensure_size (&conn->redirect, len);
 
 		if (! cherokee_buffer_is_empty (&conn->userdir)) {
 			cherokee_buffer_add_str (&conn->redirect, "/~");
 			cherokee_buffer_add_buffer (&conn->redirect, &conn->userdir);
 		}
 
-		/* Build the redirection
+		/* In case the connection has a custom Document Root directory,
+		 * it must add the web equivalent directory to the path (web_directory).
 		 */
-		if ((conn->uses_document_root) &&
-		    (! cherokee_buffer_is_empty (&conn->web_directory))) 
-		{
+		if (cherokee_connection_use_webdir (conn)) {
 			cherokee_buffer_add_buffer (&conn->redirect, &conn->web_directory);
 		}
+		
+		if (! cherokee_buffer_is_empty (&conn->host)) {
+			if (conn->socket.is_tls == TLS)
+				cherokee_buffer_add_str (&conn->redirect, "https://");
+			else
+				cherokee_buffer_add_str (&conn->redirect, "http://");
+
+			cherokee_buffer_add_buffer (&conn->redirect, &conn->host);
+		}
 
 		cherokee_buffer_add_buffer (&conn->redirect, &conn->request);
 		cherokee_buffer_add_str (&conn->redirect, "/");
@@ -612,7 +631,7 @@ build_file_list (cherokee_handler_dirlist_t *dhdl)
 	/* Clean
 	 */
 	closedir(dir);
-	cherokee_buffer_drop_endding (&conn->local_directory, conn->request.len); /* 2 */
+	cherokee_buffer_drop_ending (&conn->local_directory, conn->request.len); /* 2 */
 
 	/* Sort the file list
 	 */
@@ -678,7 +697,7 @@ read_notice_file (cherokee_handler_dirlist_t *dhdl)
 			cherokee_buffer_add (&conn->local_directory, filename, filename_len);
 			
 			ret = cherokee_buffer_read_file (&dhdl->header, conn->local_directory.buf);
-			cherokee_buffer_drop_endding (&conn->local_directory, conn->request.len + filename_len); /* undo */
+			cherokee_buffer_drop_ending (&conn->local_directory, conn->request.len + filename_len); /* undo */
 		} else {
 			ret = cherokee_buffer_read_file (&dhdl->header, filename);
 		}
@@ -782,9 +801,9 @@ static ret_t
 render_file (cherokee_handler_dirlist_t *dhdl, cherokee_buffer_t *buffer, file_entry_t *file)
 {
 	ret_t                             ret;
-	cherokee_boolean_t                is_dir;
-	cherokee_boolean_t                is_link;
 	cherokee_buffer_t                *vtmp[2];
+	cherokee_boolean_t                is_dir;
+	cherokee_boolean_t                is_link  = false;
 	char                             *alt      = NULL;
 	cherokee_buffer_t                *icon     = NULL;
 	char                             *name     = (char *) &file->info.d_name;
@@ -799,7 +818,9 @@ render_file (cherokee_handler_dirlist_t *dhdl, cherokee_buffer_t *buffer, file_e
 	VTMP_INIT_SUBST (thread, vtmp, &props->entry);
 
 	is_dir  = S_ISDIR(file->stat.st_mode);
+#ifdef S_ISLNK
 	is_link = S_ISLNK(file->stat.st_mode);
+#endif
 
 	/* Check whether it is a symlink that we should skip
 	 */
@@ -855,13 +876,15 @@ render_file (cherokee_handler_dirlist_t *dhdl, cherokee_buffer_t *buffer, file_e
 		if (is_dir) {
 			VTMP_SUBSTITUTE_TOKEN ("%size_unit%", NULL);
 			VTMP_SUBSTITUTE_TOKEN ("%size%", "-");
+		} else if (is_link) {
+			VTMP_SUBSTITUTE_TOKEN ("%size_unit%", NULL);
+			VTMP_SUBSTITUTE_TOKEN ("%size%", "link");
 		} else {
 			char *unit;
 
 			cherokee_buffer_clean (tmp);
 			cherokee_buffer_ensure_size (tmp, 8);
-
-			cherokee_strfsize (file->stat.st_size, tmp->buf);
+			cherokee_buffer_add_fsize (tmp, file->stat.st_size);
 
 			unit = tmp->buf;
 			while ((*unit >= '0')  && (*unit <= '9')) unit++;
